@@ -13,12 +13,16 @@
 
 from __future__ import annotations
 
-from typing import Callable, Dict, Iterable, List, Optional, Tuple, Type, TypeVar, Union
+import warnings
+from numbers import Number
+from typing import Callable, Dict, Iterable, List, Optional, Set, Tuple, Type, TypeVar, Union
 
 import numpy as np
 
 from braket.circuits import compiler_directives
 from braket.circuits.ascii_circuit_diagram import AsciiCircuitDiagram
+from braket.circuits.free_parameter import FreeParameter
+from braket.circuits.free_parameter_expression import FreeParameterExpression
 from braket.circuits.gate import Gate
 from braket.circuits.instruction import Instruction
 from braket.circuits.moments import Moments
@@ -33,10 +37,11 @@ from braket.circuits.noise_helpers import (
 )
 from braket.circuits.observable import Observable
 from braket.circuits.observables import TensorProduct
+from braket.circuits.parameterizable import Parameterizable
 from braket.circuits.qubit import QubitInput
 from braket.circuits.qubit_set import QubitSet, QubitSetInput
 from braket.circuits.result_type import ObservableResultType, ResultType
-from braket.circuits.unitary_calculation import calculate_unitary
+from braket.circuits.unitary_calculation import calculate_unitary, calculate_unitary_big_endian
 from braket.ir.jaqcd import Program
 
 SubroutineReturn = TypeVar(
@@ -126,6 +131,7 @@ class Circuit:
         self._qubit_observable_mapping: Dict[Union[int, Circuit._ALL_QUBITS], Observable] = {}
         self._qubit_observable_target_mapping: Dict[int, Tuple[int]] = {}
         self._qubit_observable_set = set()
+        self._parameters = set()
         self._observables_simultaneously_measurable = True
         self._has_compiler_directives = False
 
@@ -196,6 +202,16 @@ class Circuit:
         """QubitSet: Get a copy of the qubits for this circuit."""
         return QubitSet(self._moments.qubits.union(self._qubit_observable_set))
 
+    @property
+    def parameters(self) -> Set[FreeParameter]:
+        """
+        Gets a set of the parameters in the Circuit.
+
+        Returns:
+            Set[FreeParameter]: The `FreeParameters` in the Circuit.
+        """
+        return self._parameters
+
     def add_result_type(
         self,
         result_type: ResultType,
@@ -412,10 +428,32 @@ class Circuit:
             # non single qubit operator with target, add instruction with target
             instructions_to_add = [instruction.copy(target=target)]
 
+        if self._check_for_params(instruction):
+            for param in instruction.operator.parameters:
+                free_params = param.expression.free_symbols
+                for parameter in free_params:
+                    self._parameters.add(FreeParameter(parameter.name))
         self._moments.add(instructions_to_add)
 
         return self
 
+    def _check_for_params(self, instruction: Instruction) -> bool:
+        """
+        This checks for free parameters in an :class:{Instruction}. Checks children classes of
+        :class:{Parameterizable}.
+
+        Args:
+            instruction (Instruction): The instruction to check for a
+            :class:{FreeParameterExpression}.
+
+        Returns:
+            bool: Whether an object is parameterized.
+        """
+        return issubclass(type(instruction.operator), Parameterizable) and any(
+            issubclass(type(param), FreeParameterExpression)
+            for param in instruction.operator.parameters
+        )
+
     def add_circuit(
         self,
         circuit: Circuit,
@@ -773,6 +811,88 @@ class Circuit:
 
         return apply_noise_to_moments(self, noise, target_qubits, "initialization")
 
+    def make_bound_circuit(self, param_values: Dict[str, Number], strict: bool = False) -> Circuit:
+        """
+        Binds FreeParameters based upon their name and values passed in. If parameters
+        share the same name, all the parameters of that name will be set to the mapped value.
+
+        Args:
+            param_values (Dict[str, Number]):  A mapping of FreeParameter names
+                to a value to assign to them.
+            strict (bool, optional): If True, raises a ValueError if none of the FreeParameters
+                in param_values appear in the circuit. False by default."
+
+        Returns:
+            Circuit: Returns a circuit with all present parameters fixed to their respective
+            values.
+        """
+        if strict:
+            self._validate_parameters(param_values)
+        return self._use_parameter_value(param_values)
+
+    def _validate_parameters(self, parameter_values: Dict[str, Number]):
+        """
+        This runs a check to see that the parameters are in the Circuit.
+
+        Args:
+            parameter_values (Dict[str, Number]):  A mapping of FreeParameter names
+                to a value to assign to them.
+
+        Raises:
+            ValueError: If there are no parameters that match the key for the arg
+            param_values.
+        """
+        parameter_strings = set()
+        for parameter in self.parameters:
+            parameter_strings.add(str(parameter))
+        for param in parameter_values:
+            if param not in parameter_strings:
+                raise ValueError(f"No parameter in the circuit named: {param}")
+
+    def _use_parameter_value(self, param_values: Dict[str, Number]) -> Circuit:
+        """
+        Creates a Circuit that uses the parameter values passed in.
+
+        Args:
+            param_values (Dict[str, Number]): A mapping of FreeParameter names
+                to a value to assign to them.
+
+        Returns:
+            Circuit: A Circuit with specified parameters swapped for their
+            values.
+
+        """
+        fixed_circ = Circuit()
+        for val in param_values.values():
+            self._validate_parameter_value(val)
+        for instruction in self.instructions:
+            if self._check_for_params(instruction):
+                fixed_circ.add(
+                    Instruction(
+                        instruction.operator.bind_values(**param_values), target=instruction.target
+                    )
+                )
+            else:
+                fixed_circ.add(instruction)
+        fixed_circ.add(self.result_types)
+        return fixed_circ
+
+    @staticmethod
+    def _validate_parameter_value(val):
+        """
+        Validates the value being used is a Number.
+
+        Args:
+            val: The value be verified.
+
+        Raises:
+            ValueError: If the value is not a Number
+        """
+        if not isinstance(val, Number):
+            raise ValueError(
+                f"Parameters can only be assigned numeric values. " f"Invalid inputs: {val}"
+            )
+
     def apply_readout_noise(
         self,
         noise: Union[Type[Noise], Iterable[Type[Noise]]],
@@ -944,8 +1064,8 @@ class Circuit:
         )
 
     def as_unitary(self) -> np.ndarray:
-        """
-        Returns the unitary matrix representation of the entire circuit.
+        r"""
+        Returns the unitary matrix representation, in little endian format, of the entire circuit.
         *Note*: The performance of this method degrades with qubit count. It might be slow for
         qubit count > 10.
 
@@ -954,6 +1074,12 @@ class Circuit:
                 circuit as a unitary. *Note*: For an empty circuit, an empty numpy array is
                 returned (`array([], dtype=complex128)`)
 
+        Warnings:
+            This method has been deprecated, please use to_unitary() instead.
+            The unitary returned by this method is *little-endian*; the first qubit in the circuit
+            is the _least_ significant. For example, a circuit `Circuit().h(0).x(1)` will yield the
+            unitary :math:`X(1) \otimes H(0)`.
+
         Raises:
             TypeError: If circuit is not composed only of `Gate` instances,
                 i.e. a circuit with `Noise` operators will raise this error.
@@ -970,6 +1096,12 @@ class Circuit:
                    [ 0.70710678+0.j, -0.70710678+0.j,  0.        +0.j,
                      0.        +0.j]])
         """
+        warnings.warn(
+            "Matrix returned will have qubits in little-endian order; "
+            "This method has been deprecated. Please use to_unitary() instead.",
+            category=DeprecationWarning,
+        )
+
         qubits = self.qubits
         if not qubits:
             return np.zeros(0, dtype=complex)
@@ -977,6 +1109,40 @@ class Circuit:
 
         return calculate_unitary(qubit_count, self.instructions)
 
+    def to_unitary(self) -> np.ndarray:
+        """
+        Returns the unitary matrix representation of the entire circuit.
+
+        Note:
+            The performance of this method degrades with qubit count. It might be slow for
+            `qubit count` > 10.
+
+        Returns:
+            np.ndarray: A numpy array with shape (2^qubit_count, 2^qubit_count) representing the
+            circuit as a unitary. For an empty circuit, an empty numpy array is returned
+            (`array([], dtype=complex)`)
+
+        Raises:
+            TypeError: If circuit is not composed only of `Gate` instances,
+                i.e. a circuit with `Noise` operators will raise this error.
+
+        Examples:
+            >>> circ = Circuit().h(0).cnot(0, 1)
+            >>> circ.to_unitary()
+            array([[ 0.70710678+0.j,  0.        +0.j,  0.70710678+0.j,
+                     0.        +0.j],
+                   [ 0.        +0.j,  0.70710678+0.j,  0.        +0.j,
+                     0.70710678+0.j],
+                   [ 0.        +0.j,  0.70710678+0.j,  0.        +0.j,
+                    -0.70710678+0.j],
+                   [ 0.70710678+0.j,  0.        +0.j, -0.70710678+0.j,
+                     0.        +0.j]])
+        """
+        qubits = self.qubits
+        if not qubits:
+            return np.zeros(0, dtype=complex)
+        return calculate_unitary_big_endian(self.instructions, qubits)
+
     @property
     def qubits_frozen(self) -> bool:
         """bool: Whether the circuit's qubits are frozen, that is, cannot be remapped.
@@ -1044,6 +1210,26 @@ class Circuit:
             )
         return NotImplemented
 
+    def __call__(self, arg=None, **kwargs) -> Circuit:
+        """
+        Implements the call function to easily make a bound Circuit.
+
+        Args:
+            arg: A value to bind to all parameters. Defaults to None and
+                can be overridden if the parameter is in kwargs.
+            **kwargs: the named parameters to have their value bound.
+
+        Returns:
+            Circuit: A circuit with the specified parameters bound.
+        """
+        param_values = dict()
+        if arg is not None:
+            for param in self.parameters:
+                param_values[str(param)] = arg
+        for key, val in kwargs.items():
+            param_values[str(key)] = val
+        return self.make_bound_circuit(param_values)
+
 
 def subroutine(register=False):
     """
