@@ -985,3 +985,14 @@ def test_exceptions_in_all_device_regions(aws_session):
     )
     with pytest.raises(ClientError, match=error_message):
         AwsQuantumJob._initialize_session(aws_session, device_arn, logger)
+
+
+def test_bad_arn_format(aws_session):
+    logger = logging.getLogger(__name__)
+    device_not_found = (
+        "Device ARN is not a valid format: bad-arn-format. For valid Braket ARNs, "
+        "see 'https://docs.aws.amazon.com/braket/latest/developerguide/braket-devices.html'"
+    )
+
+    with pytest.raises(ValueError, match=device_not_found):
+        AwsQuantumJob._initialize_session(aws_session, "bad-arn-format", logger)
