@@ -834,6 +834,7 @@ def test_get_default_jobs_role(aws_session, job_role_arn, job_role_name):
                 "IsTruncated": True,
                 "Marker": "resp-marker",
             },
+            {"PathPrefix": "/service-role/"},
         )
         stub.add_response(
             "list_roles",
@@ -849,7 +850,7 @@ def test_get_default_jobs_role(aws_session, job_role_arn, job_role_name):
                 ],
                 "IsTruncated": False,
             },
-            {"Marker": "resp-marker"},
+            {"Marker": "resp-marker", "PathPrefix": "/service-role/"},
         )
         aws_session._iam = iam_client
         assert aws_session.get_default_jobs_role() == job_role_arn
@@ -874,6 +875,7 @@ def test_get_default_jobs_role_not_found(aws_session, job_role_arn, job_role_nam
                 "IsTruncated": True,
                 "Marker": "resp-marker",
             },
+            {"PathPrefix": "/service-role/"},
         )
         stub.add_response(
             "list_roles",
@@ -889,7 +891,7 @@ def test_get_default_jobs_role_not_found(aws_session, job_role_arn, job_role_nam
                 ],
                 "IsTruncated": False,
             },
-            {"Marker": "resp-marker"},
+            {"Marker": "resp-marker", "PathPrefix": "/service-role/"},
         )
         aws_session._iam = iam_client
         with pytest.raises(RuntimeError):
