@@ -20,6 +20,7 @@ import braket.ir.jaqcd as jaqcd
 from braket.circuits import (
     AsciiCircuitDiagram,
     Circuit,
+    FreeParameter,
     Gate,
     Instruction,
     Moments,
@@ -104,6 +105,64 @@ def test_equality():
     assert circ_1 != non_circ
 
 
+def test_call():
+    alpha = FreeParameter("alpha")
+    theta = FreeParameter("theta")
+    circ = Circuit().h(0).rx(angle=theta, target=1).ry(angle=alpha, target=0)
+    new_circ = circ(theta=1, alpha=0)
+    expected = Circuit().h(0).rx(angle=1, target=1).ry(angle=0, target=0)
+    assert new_circ == expected and not new_circ.parameters
+
+
+def test_call_with_result_type(prob):
+    alpha = FreeParameter("alpha")
+    theta = FreeParameter("theta")
+    circ = Circuit().h(0).rx(angle=theta, target=1).ry(angle=alpha, target=0).add_result_type(prob)
+    new_circ = circ(theta=1, alpha=0)
+    expected = Circuit().h(0).rx(angle=1, target=1).ry(angle=0, target=0).add_result_type(prob)
+
+    assert new_circ == expected and not new_circ.parameters
+    assert new_circ.observables_simultaneously_measurable
+    assert list(new_circ.result_types) == [prob]
+
+
+def test_call_one_param_not_bound():
+    alpha = FreeParameter("alpha")
+    theta = FreeParameter("theta")
+    circ = Circuit().h(0).rx(angle=theta, target=1).ry(angle=alpha, target=0)
+    new_circ = circ(theta=1)
+    expected_circ = Circuit().h(0).rx(angle=1, target=1).ry(angle=alpha, target=0)
+    expected_parameters = set()
+    expected_parameters.add(alpha)
+
+    assert new_circ == expected_circ and new_circ.parameters == expected_parameters
+
+
+def test_call_with_default_parameter_val():
+    alpha = FreeParameter("alpha")
+    beta = FreeParameter("beta")
+    theta = FreeParameter("theta")
+    gamma = FreeParameter("gamma")
+    circ = (
+        Circuit()
+        .h(0)
+        .rx(angle=theta, target=1)
+        .ry(angle=alpha, target=0)
+        .ry(angle=beta, target=2)
+        .rx(angle=gamma, target=1)
+    )
+    new_circ = circ(np.pi, theta=1, alpha=0)
+    expected = (
+        Circuit()
+        .h(0)
+        .rx(angle=1, target=1)
+        .ry(angle=0, target=0)
+        .ry(angle=np.pi, target=2)
+        .rx(angle=np.pi, target=1)
+    )
+    assert new_circ == expected and not new_circ.parameters
+
+
 def test_add_result_type_default(prob):
     circ = Circuit().add_result_type(prob)
     assert circ.observables_simultaneously_measurable
@@ -607,6 +666,13 @@ def test_as_unitary_noise_raises_error(circuit):
     circuit.as_unitary()
 
 
+@pytest.mark.xfail(raises=TypeError)
+def test_as_unitary_parameterized():
+    theta = FreeParameter("theta")
+    circ = Circuit().rx(angle=theta, target=0)
+    assert np.allclose(circ.as_unitary())
+
+
 def test_as_unitary_noise_not_apply_returns_expected_unitary(recwarn):
     circuit = (
         Circuit()
@@ -946,6 +1012,399 @@ def test_as_unitary_one_gate_returns_expected_unitary(circuit, expected_unitary)
     assert np.allclose(circuit.as_unitary(), expected_unitary)
 
 
+def test_to_unitary_empty_instructions_returns_empty_array():
+    circ = Circuit()
+    circ.to_unitary() == []
+
+
+@pytest.mark.parametrize(
+    "circuit",
+    [
+        (Circuit().phaseshift(0, 0.15).apply_gate_noise(noise.Noise.BitFlip(probability=0.1))),
+        (Circuit().cnot(1, 0).apply_gate_noise(noise.Noise.TwoQubitDepolarizing(probability=0.1))),
+        (
+            Circuit()
+            .x(1)
+            .i(2)
+            .apply_gate_noise(noise.Noise.BitFlip(probability=0.1), target_qubits=[1])
+        ),
+        (
+            Circuit()
+            .x(1)
+            .i(2)
+            .apply_gate_noise(noise.Noise.BitFlip(probability=0.1), target_qubits=[2])
+        ),
+        (Circuit().x(1).i(2).apply_gate_noise(noise.Noise.BitFlip(probability=0.1))),
+        (Circuit().x(1).apply_gate_noise(noise.Noise.BitFlip(probability=0.1)).i(2)),
+        (
+            Circuit()
+            .y(1)
+            .z(2)
+            .apply_gate_noise(noise.Noise.BitFlip(probability=0.1), target_qubits=[1])
+        ),
+        (
+            Circuit()
+            .y(1)
+            .z(2)
+            .apply_gate_noise(noise.Noise.BitFlip(probability=0.1), target_qubits=[2])
+        ),
+        (Circuit().y(1).z(2).apply_gate_noise(noise.Noise.BitFlip(probability=0.1))),
+        (Circuit().y(1).apply_gate_noise(noise.Noise.BitFlip(probability=0.1)).z(2)),
+        (
+            Circuit()
+            .cphaseshift(2, 1, 0.15)
+            .si(3)
+            .apply_gate_noise(
+                noise.Noise.TwoQubitDepolarizing(probability=0.1), target_qubits=[1, 2]
+            )
+        ),
+        (
+            Circuit()
+            .cphaseshift(2, 1, 0.15)
+            .apply_gate_noise(noise.Noise.TwoQubitDepolarizing(probability=0.1))
+            .si(3)
+        ),
+    ],
+)
+@pytest.mark.xfail(raises=TypeError)
+def test_to_unitary_noise_raises_error(circuit):
+    circuit.to_unitary()
+
+
+@pytest.mark.xfail(raises=TypeError)
+def test_to_unitary_parameterized():
+    theta = FreeParameter("theta")
+    circ = Circuit().rx(angle=theta, target=0)
+    assert np.allclose(circ.to_unitary())
+
+
+def test_to_unitary_noise_not_apply_returns_expected_unitary(recwarn):
+    circuit = (
+        Circuit()
+        .cphaseshift(1, 2, 0.15)
+        .si(3)
+        .apply_gate_noise(noise.Noise.TwoQubitDepolarizing(probability=0.1), target_qubits=[1, 3])
+    )
+
+    assert len(recwarn) == 1
+    assert str(recwarn[0].message).startswith("Noise is not applied to any gate")
+
+    assert np.allclose(
+        circuit.to_unitary(),
+        np.kron(gates.CPhaseShift(0.15).to_matrix(), gates.Si().to_matrix()),
+    )
+
+
+@pytest.mark.parametrize(
+    "circuit,expected_unitary",
+    [
+        (Circuit().h(0), gates.H().to_matrix()),
+        (Circuit().h(0).add_result_type(ResultType.Probability(target=[0])), gates.H().to_matrix()),
+        (Circuit().h(1), gates.H().to_matrix()),
+        (Circuit().h(2), gates.H().to_matrix()),
+        (Circuit().x(0), gates.X().to_matrix()),
+        (Circuit().y(0), gates.Y().to_matrix()),
+        (Circuit().z(0), gates.Z().to_matrix()),
+        (Circuit().s(0), gates.S().to_matrix()),
+        (Circuit().si(0), gates.Si().to_matrix()),
+        (Circuit().t(0), gates.T().to_matrix()),
+        (Circuit().ti(0), gates.Ti().to_matrix()),
+        (Circuit().v(0), gates.V().to_matrix()),
+        (Circuit().vi(0), gates.Vi().to_matrix()),
+        (Circuit().rx(0, 0.15), gates.Rx(0.15).to_matrix()),
+        (Circuit().ry(0, 0.15), gates.Ry(0.15).to_matrix()),
+        (Circuit().rz(0, 0.15), gates.Rz(0.15).to_matrix()),
+        (Circuit().phaseshift(0, 0.15), gates.PhaseShift(0.15).to_matrix()),
+        (Circuit().cnot(0, 1), gates.CNot().to_matrix()),
+        (Circuit().cnot(0, 1).add_result_type(ResultType.StateVector()), gates.CNot().to_matrix()),
+        (Circuit().cnot(2, 4), gates.CNot().to_matrix()),
+        (Circuit().swap(0, 1), gates.Swap().to_matrix()),
+        (Circuit().swap(1, 0), gates.Swap().to_matrix()),
+        (Circuit().iswap(0, 1), gates.ISwap().to_matrix()),
+        (Circuit().iswap(1, 0), gates.ISwap().to_matrix()),
+        (Circuit().pswap(0, 1, 0.15), gates.PSwap(0.15).to_matrix()),
+        (Circuit().pswap(1, 0, 0.15), gates.PSwap(0.15).to_matrix()),
+        (Circuit().xy(0, 1, 0.15), gates.XY(0.15).to_matrix()),
+        (Circuit().xy(1, 0, 0.15), gates.XY(0.15).to_matrix()),
+        (Circuit().cphaseshift(0, 1, 0.15), gates.CPhaseShift(0.15).to_matrix()),
+        (Circuit().cphaseshift00(0, 1, 0.15), gates.CPhaseShift00(0.15).to_matrix()),
+        (Circuit().cphaseshift01(0, 1, 0.15), gates.CPhaseShift01(0.15).to_matrix()),
+        (Circuit().cphaseshift10(0, 1, 0.15), gates.CPhaseShift10(0.15).to_matrix()),
+        (Circuit().cy(0, 1), gates.CY().to_matrix()),
+        (Circuit().cz(0, 1), gates.CZ().to_matrix()),
+        (Circuit().xx(0, 1, 0.15), gates.XX(0.15).to_matrix()),
+        (Circuit().yy(0, 1, 0.15), gates.YY(0.15).to_matrix()),
+        (Circuit().zz(0, 1, 0.15), gates.ZZ(0.15).to_matrix()),
+        (Circuit().ccnot(0, 1, 2), gates.CCNot().to_matrix()),
+        (
+            Circuit()
+            .ccnot(0, 1, 2)
+            .add_result_type(ResultType.Expectation(observable=Observable.Y(), target=[1])),
+            gates.CCNot().to_matrix(),
+        ),
+        (Circuit().ccnot(0, 1, 2), gates.CCNot().to_matrix()),
+        (Circuit().cswap(0, 1, 2), gates.CSwap().to_matrix()),
+        (Circuit().cswap(0, 2, 1), gates.CSwap().to_matrix()),
+        (Circuit().h(1), gates.H().to_matrix()),
+        (Circuit().x(1).i(2), np.kron(gates.X().to_matrix(), np.eye(2))),
+        (Circuit().y(1).z(2), np.kron(gates.Y().to_matrix(), gates.Z().to_matrix())),
+        (Circuit().rx(1, 0.15), gates.Rx(0.15).to_matrix()),
+        (Circuit().ry(1, 0.15).i(2), np.kron(gates.Ry(0.15).to_matrix(), np.eye(2))),
+        (Circuit().rz(1, 0.15).s(2), np.kron(gates.Rz(0.15).to_matrix(), gates.S().to_matrix())),
+        (Circuit().pswap(1, 2, 0.15), gates.PSwap(0.15).to_matrix()),
+        (Circuit().pswap(2, 1, 0.15), gates.PSwap(0.15).to_matrix()),
+        (Circuit().xy(1, 2, 0.15).i(3), np.kron(gates.XY(0.15).to_matrix(), np.eye(2))),
+        (Circuit().xy(2, 1, 0.15).i(3), np.kron(gates.XY(0.15).to_matrix(), np.eye(2))),
+        (
+            Circuit().cphaseshift(1, 2, 0.15).si(3),
+            np.kron(gates.CPhaseShift(0.15).to_matrix(), gates.Si().to_matrix()),
+        ),
+        (Circuit().ccnot(1, 2, 3), gates.CCNot().to_matrix()),
+        (Circuit().ccnot(2, 1, 3), gates.CCNot().to_matrix()),
+        (Circuit().cswap(1, 2, 3).i(4), np.kron(gates.CSwap().to_matrix(), np.eye(2))),
+        (Circuit().cswap(1, 3, 2).i(4), np.kron(gates.CSwap().to_matrix(), np.eye(2))),
+        (Circuit().cswap(1, 2, 3).t(4), np.kron(gates.CSwap().to_matrix(), gates.T().to_matrix())),
+        (Circuit().cswap(1, 3, 2).t(4), np.kron(gates.CSwap().to_matrix(), gates.T().to_matrix())),
+        (Circuit().h(0).h(0), gates.I().to_matrix()),
+        (Circuit().h(0).x(0), np.dot(gates.X().to_matrix(), gates.H().to_matrix())),
+        (Circuit().x(0).h(0), np.dot(gates.H().to_matrix(), gates.X().to_matrix())),
+        (
+            Circuit().y(0).z(1).cnot(0, 1),
+            np.dot(gates.CNot().to_matrix(), np.kron(gates.Y().to_matrix(), gates.Z().to_matrix())),
+        ),
+        (
+            Circuit().z(0).y(1).cnot(0, 1),
+            np.dot(gates.CNot().to_matrix(), np.kron(gates.Z().to_matrix(), gates.Y().to_matrix())),
+        ),
+        (
+            Circuit().y(0).z(1).cnot(0, 1).cnot(1, 2),
+            np.dot(
+                np.dot(
+                    np.kron(np.eye(2), gates.CNot().to_matrix()),
+                    np.kron(gates.CNot().to_matrix(), np.eye(2)),
+                ),
+                np.kron(np.kron(gates.Y().to_matrix(), gates.Z().to_matrix()), np.eye(2)),
+            ),
+        ),
+        (
+            Circuit().z(0).y(1).cnot(0, 1).ccnot(0, 1, 2),
+            np.dot(
+                np.dot(
+                    gates.CCNot().to_matrix(),
+                    np.kron(gates.CNot().to_matrix(), np.eye(2)),
+                ),
+                np.kron(np.kron(gates.Z().to_matrix(), gates.Y().to_matrix()), np.eye(2)),
+            ),
+        ),
+        (
+            Circuit().cnot(1, 0),
+            np.array(
+                [
+                    [1.0, 0.0, 0.0, 0.0],
+                    [0.0, 0.0, 0.0, 1.0],
+                    [0.0, 0.0, 1.0, 0.0],
+                    [0.0, 1.0, 0.0, 0.0],
+                ],
+                dtype=complex,
+            ),
+        ),
+        (
+            Circuit().ccnot(1, 2, 0),
+            np.array(
+                [
+                    [1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
+                    [0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
+                    [0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0],
+                    [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0],
+                    [0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0],
+                    [0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0],
+                    [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0],
+                    [0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0],
+                ],
+                dtype=complex,
+            ),
+        ),
+        (
+            Circuit().ccnot(2, 1, 0),
+            np.array(
+                [
+                    [1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
+                    [0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
+                    [0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0],
+                    [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0],
+                    [0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0],
+                    [0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0],
+                    [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0],
+                    [0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0],
+                ],
+                dtype=complex,
+            ),
+        ),
+        (
+            Circuit().ccnot(0, 2, 1),
+            np.array(
+                [
+                    [1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
+                    [0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
+                    [0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0],
+                    [0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0],
+                    [0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0],
+                    [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0],
+                    [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0],
+                    [0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0],
+                ],
+                dtype=complex,
+            ),
+        ),
+        (
+            Circuit().ccnot(2, 0, 1),
+            np.array(
+                [
+                    [1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
+                    [0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
+                    [0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0],
+                    [0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0],
+                    [0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0],
+                    [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0],
+                    [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0],
+                    [0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0],
+                ],
+                dtype=complex,
+            ),
+        ),
+        (
+            Circuit().s(0).v(1).cnot(0, 1).cnot(2, 1),
+            np.dot(
+                np.dot(
+                    np.dot(
+                        np.kron(
+                            np.eye(2),
+                            np.array(
+                                [
+                                    [1.0, 0.0, 0.0, 0.0],
+                                    [0.0, 0.0, 0.0, 1.0],
+                                    [0.0, 0.0, 1.0, 0.0],
+                                    [0.0, 1.0, 0.0, 0.0],
+                                ],
+                                dtype=complex,
+                            ),
+                        ),
+                        np.kron(
+                            np.array(
+                                [
+                                    [1.0, 0.0, 0.0, 0.0],
+                                    [0.0, 1.0, 0.0, 0.0],
+                                    [0.0, 0.0, 0.0, 1.0],
+                                    [0.0, 0.0, 1.0, 0.0],
+                                ],
+                                dtype=complex,
+                            ),
+                            np.eye(2),
+                        ),
+                    ),
+                    np.kron(np.kron(np.eye(2), gates.V().to_matrix()), np.eye(2)),
+                ),
+                np.kron(gates.S().to_matrix(), np.eye(4)),
+            ),
+        ),
+        (
+            Circuit().z(0).y(1).cnot(1, 0).ccnot(2, 1, 0),
+            np.dot(
+                np.dot(
+                    np.dot(
+                        np.array(
+                            [
+                                [1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
+                                [0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
+                                [0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0],
+                                [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0],
+                                [0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0],
+                                [0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0],
+                                [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0],
+                                [0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0],
+                            ],
+                            dtype=complex,
+                        ),
+                        np.kron(
+                            np.array(
+                                [
+                                    [1.0, 0.0, 0.0, 0.0],
+                                    [0.0, 0.0, 0.0, 1.0],
+                                    [0.0, 0.0, 1.0, 0.0],
+                                    [0.0, 1.0, 0.0, 0.0],
+                                ],
+                                dtype=complex,
+                            ),
+                            np.eye(2),
+                        ),
+                    ),
+                    np.kron(np.kron(np.eye(2), gates.Y().to_matrix()), np.eye(2)),
+                ),
+                np.kron(gates.Z().to_matrix(), np.eye(4)),
+            ),
+        ),
+        (
+            Circuit().z(0).y(1).cnot(1, 0).ccnot(2, 0, 1),
+            np.dot(
+                np.dot(
+                    np.dot(
+                        np.array(
+                            [
+                                [1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
+                                [0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0],
+                                [0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0],
+                                [0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0],
+                                [0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0],
+                                [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0],
+                                [0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0],
+                                [0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0],
+                            ],
+                            dtype=complex,
+                        ),
+                        np.kron(
+                            np.array(
+                                [
+                                    [1.0, 0.0, 0.0, 0.0],
+                                    [0.0, 0.0, 0.0, 1.0],
+                                    [0.0, 0.0, 1.0, 0.0],
+                                    [0.0, 1.0, 0.0, 0.0],
+                                ],
+                                dtype=complex,
+                            ),
+                            np.eye(2),
+                        ),
+                    ),
+                    np.kron(np.kron(np.eye(2), gates.Y().to_matrix()), np.eye(2)),
+                ),
+                np.kron(gates.Z().to_matrix(), np.eye(4)),
+            ),
+        ),
+    ],
+)
+def test_to_matrix_one_gate_returns_expected_unitary(circuit, expected_unitary):
+    assert np.allclose(circuit.to_unitary(), expected_unitary)
+
+
+def test_circuit_with_symbol():
+    theta = FreeParameter("theta")
+
+    circ = (
+        Circuit()
+        .ry(angle=theta, target=0)
+        .ry(angle=theta, target=1)
+        .ry(angle=theta, target=2)
+        .ry(angle=theta, target=3)
+    )
+    expected = (
+        Circuit()
+        .ry(angle=theta, target=0)
+        .ry(angle=theta, target=1)
+        .ry(angle=theta, target=2)
+        .ry(angle=theta, target=3)
+    )
+    assert circ == expected
+
+
 def test_basis_rotation_instructions_all():
     circ = Circuit().h(0).cnot(0, 1).sample(observable=Observable.Y())
     expected = [
@@ -1306,3 +1765,205 @@ def test_diagram(h):
 
     assert h.diagram(mock_diagram) == expected
     mock_diagram.build_diagram.assert_called_with(h)
+
+
+def test_add_parameterized_check_true():
+    theta = FreeParameter("theta")
+    circ = (
+        Circuit()
+        .ry(angle=theta, target=0)
+        .ry(angle=theta, target=1)
+        .ry(angle=theta, target=2)
+        .ry(angle=theta, target=3)
+    )
+    expected = set()
+    expected.add(theta)
+
+    assert circ.parameters == expected
+
+
+def test_add_parameterized_instr_parameterized_circ_check_true():
+    theta = FreeParameter("theta")
+    alpha = FreeParameter("alpha")
+    alpha2 = FreeParameter("alpha")
+    circ = Circuit().ry(angle=theta, target=0).ry(angle=alpha2, target=1).ry(angle=theta, target=2)
+    circ.add_instruction(Instruction(Gate.Ry(alpha), 3))
+    expected = set()
+    expected.add(theta)
+    expected.add(alpha)
+
+    assert circ.parameters == expected
+
+
+def test_add_non_parameterized_instr_parameterized_check_true():
+    theta = FreeParameter("theta")
+    circ = Circuit().ry(angle=theta, target=0).ry(angle=theta, target=1).ry(angle=theta, target=2)
+    circ.add_instruction(Instruction(Gate.Ry(0.1), 3))
+    expected = set()
+    expected.add(theta)
+
+    assert circ.parameters == expected
+
+
+def test_add_circ_parameterized_check_true():
+    theta = FreeParameter("theta")
+    circ = Circuit().ry(angle=1, target=0).add_circuit(Circuit().ry(angle=theta, target=0))
+
+    expected = set()
+    expected.add(theta)
+
+    assert circ.parameters == expected
+
+
+def test_add_circ_not_parameterized_check_true():
+    theta = FreeParameter("theta")
+    circ = Circuit().ry(angle=theta, target=0).add_circuit(Circuit().ry(angle=0.1, target=0))
+
+    expected = set()
+    expected.add(theta)
+
+    assert circ.parameters == expected
+
+
+@pytest.mark.parametrize(
+    "input_circ",
+    [
+        (Circuit().ry(angle=1, target=0).ry(angle=2, target=1)),
+        (Circuit().ry(angle=1, target=0).add_circuit(Circuit().ry(angle=2, target=0))),
+    ],
+)
+def test_parameterized_check_false(input_circ):
+    circ = input_circ
+    expected = 0
+
+    assert len(circ.parameters) == expected
+
+
+def test_parameters():
+    theta = FreeParameter("theta")
+    circ = Circuit().ry(angle=theta, target=0).ry(angle=theta, target=1).ry(angle=theta, target=2)
+    expected = set()
+    expected.add(theta)
+
+    assert circ.parameters == expected
+
+
+def test_no_parameters():
+    circ = Circuit().ry(angle=0.12, target=0).ry(angle=0.25, target=1).ry(angle=0.6, target=2)
+    expected = set()
+
+    assert circ.parameters == expected
+
+
+def test_make_bound_circuit_strict():
+    theta = FreeParameter("theta")
+    input_val = np.pi
+    circ = Circuit().ry(angle=theta, target=0).ry(angle=theta, target=1).ry(angle=theta, target=2)
+    circ_new = circ.make_bound_circuit({"theta": input_val}, strict=True)
+    expected = (
+        Circuit().ry(angle=np.pi, target=0).ry(angle=np.pi, target=1).ry(angle=np.pi, target=2)
+    )
+
+    assert circ_new == expected
+
+
+def test_make_bound_circuit_strict_false():
+    input_val = np.pi
+    theta = FreeParameter("theta")
+    param_superset = {"theta": input_val, "alpha": input_val, "beta": input_val}
+    circ = Circuit().ry(angle=theta, target=0).ry(angle=theta, target=1).ry(angle=theta, target=2)
+    circ_new = circ.make_bound_circuit(param_superset)
+    expected = (
+        Circuit().ry(angle=np.pi, target=0).ry(angle=np.pi, target=1).ry(angle=np.pi, target=2)
+    )
+
+    assert circ_new == expected
+
+
+def test_make_bound_circuit_multiple():
+    theta = FreeParameter("theta")
+    alpha = FreeParameter("alpha")
+    input_val = np.pi
+    circ = Circuit().ry(angle=theta, target=0).ry(angle=theta, target=1).ry(angle=alpha, target=2)
+    circ_new = circ.make_bound_circuit({"theta": input_val, "alpha": input_val})
+    expected = (
+        Circuit().ry(angle=np.pi, target=0).ry(angle=np.pi, target=1).ry(angle=np.pi, target=2)
+    )
+
+    assert circ_new == expected
+
+
+def test_make_bound_circuit_partial_bind():
+    theta = FreeParameter("theta")
+    alpha = FreeParameter("alpha")
+    input_val = np.pi
+    circ = Circuit().ry(angle=theta, target=0).ry(angle=theta, target=1).ry(angle=alpha, target=2)
+    circ_new = circ.make_bound_circuit({"theta": input_val})
+    expected_circ = (
+        Circuit().ry(angle=np.pi, target=0).ry(angle=np.pi, target=1).ry(angle=alpha, target=2)
+    )
+    expected_parameters = set()
+    expected_parameters.add(alpha)
+
+    assert circ_new == expected_circ and circ_new.parameters == expected_parameters
+
+
+@pytest.mark.xfail(raises=ValueError)
+def test_make_bound_circuit_non_existent_param():
+    theta = FreeParameter("theta")
+    input_val = np.pi
+    circ = Circuit().ry(angle=theta, target=0).ry(angle=theta, target=1).ry(angle=theta, target=2)
+    circ.make_bound_circuit({"alpha": input_val}, strict=True)
+
+
+@pytest.mark.xfail(raises=ValueError)
+def test_make_bound_circuit_bad_value():
+    theta = FreeParameter("theta")
+    input_val = "invalid"
+    circ = Circuit().ry(angle=theta, target=0).ry(angle=theta, target=1).ry(angle=theta, target=2)
+    circ.make_bound_circuit({"theta": input_val})
+
+
+def test_circuit_with_expr():
+    theta = FreeParameter("theta")
+    alpha = FreeParameter("alpha")
+    circ = (
+        Circuit()
+        .ry(angle=theta * 2 + theta, target=0)
+        .rx(angle=(alpha + theta + 2 * alpha * theta), target=2)
+        .rz(angle=theta, target=1)
+    )
+    circ.add_instruction(Instruction(Gate.Ry(alpha), 3))
+
+    new_circ = circ(theta=1, alpha=np.pi)
+    expected = (
+        Circuit()
+        .ry(angle=3, target=0)
+        .rx(angle=(3 * np.pi + 1), target=2)
+        .rz(angle=1, target=1)
+        .ry(angle=np.pi, target=3)
+    )
+
+    assert new_circ == expected
+
+
+def test_circuit_with_expr_not_fully_bound():
+    theta = FreeParameter("theta")
+    alpha = FreeParameter("alpha")
+    circ = (
+        Circuit()
+        .ry(angle=theta * 2 + theta, target=0)
+        .rx(angle=(alpha + theta + 2 * alpha * theta), target=2)
+        .rz(angle=theta, target=1)
+    )
+    circ.add_instruction(Instruction(Gate.Ry(alpha), 3))
+
+    new_circ = circ(theta=1)
+    expected = (
+        Circuit()
+        .ry(angle=3, target=0)
+        .rx(angle=(3 * alpha + 1), target=2)
+        .rz(angle=1, target=1)
+        .ry(angle=alpha, target=3)
+    )
+    assert new_circ == expected
